#include "ShaderConstants.fxh"
#include ".Assets/pre.function"
#include "util.fxh"

struct PS_Input
{
    float4 position : SV_Position;
    float3 ppos : PPOS;
    float2 uv : TEXCOORD_0_FB_MSAA;
};

struct PS_Output
{
    float4 color : SV_Target;
};

ROOT_SIGNATURE
void main(in PS_Input PSInput, out PS_Output PSOutput)
{
DetectTime at;
   at.rain = clamp((1.0 - pow(FOG_CONTROL.y + 0.1, 5.0))*2.0, 0.0, 1.0);
   at.dusk = clamp(((1.5 - FOG_COLOR.r*2.0) + (1.0 - FOG_COLOR.b)) * (1.0 - dot(FOG_COLOR.rgb, float3(0.35, 0.35, 0.35))) * FOG_COLOR.g*2.0, 0.0, 1.0) * (1.0 - at.rain);
   at.night = clamp(1.0 - FOG_COLOR.g * (1.5 - at.dusk)*2.5, 0.0, 1.0);

   float3 pos = PSInput.ppos;

   pos *= lerp(lerp(1.4, 2.0, at.dusk), 0.8, at.night);

   float fix0 = max(0.5 * sunNmoon(pos*0.7, 15.0, 40.0), 0.0);
      fix0 = pow(fix0, 1.5);
   float fix1 = max(0.5 - length(pos), 0.0);
      fix1 = pow(fix1, 2.0);
   float fix2 = pow(fix1, 2.8);

   float bug0 = sunNmoon(pos, 15.0, 40.0);

   float3 sun0 = float3(1.0, 1.0, 1.0);
   float3 sun1 = lerp(float3(1.5, 0.6, -0.2), float3(1.0, 1.0, 1.0), at.night) * fix0;
   float3 sun2 = fix1 * lerp(float3(1.0, 0.7, -0.1), float3(1.0, 1.0, 1.0), at.night);

   float4 bugactive = float4(lerp(sun1, sun0, bug0), 1.0);
   float4 fixactive = max(float4(lerp(sun2, sun2+sun2, fix2), 1.0), bugactive) * 0.9;

   PSOutput.color = bugactive;
   PSOutput.color += fixactive;
   PSOutput.color.a = 1.0 * (1.1 - at.rain);

#ifdef WINDOWSMR_MAGICALPHA
    // Set the magic MR value alpha value so that this content pops over layers
    PSOutput.color.a = 133.0f / 255.0f;
#endif
}
