#include "ShaderConstants.fxh"
#include "custom/presetSetting.fxh"
#include ".Assets/pre.function"
#include ".Assets/lib.function"
#include "util.fxh"

struct PS_Input
{
	float4 position : SV_Position;
	float4 fogCoIor : Fog_Ground;
	float3 ppos : PPOS;

#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
#endif

#ifdef FOG
	float4 fogColor : FOG_COLOR;
#endif
};

struct PS_Output
{
	float4 color : SV_Target;
};

ROOT_SIGNATURE
void main(in PS_Input PSInput, out PS_Output PSOutput)
{
#ifdef BYPASS_PIXEL_SHADER
    PSOutput.color = float4(0.0f, 0.0f, 0.0f, 0.0f);
    return;
#else

#if USE_TEXEL_AA
	float4 diffuse = texture2D_AA(TEXTURE_0, TextureSampler0, PSInput.uv0 );
#else
	float4 diffuse = TEXTURE_0.Sample(TextureSampler0, PSInput.uv0);
#endif
	
	DetectTime at;
	  at.rain = clamp((1.0 - pow(FOG_CONTROL.y+0.1,5.0))*2.0, 0.0, 1.0);
	  at.dusk = clamp(((1.5-FOG_COLOR.r*2.0)+(1.0-FOG_COLOR.b))*(1.0-dot(FOG_COLOR.rgb,float3(0.35, 0.35, 0.35)))*FOG_COLOR.g*2.0, 0.0, 1.0)*(1.0-at.rain);
	  at.night = clamp(1.0-FOG_COLOR.g*(1.5-at.dusk)*2.5, 0.0, 1.0);

#ifdef SEASONS_FAR
	diffuse.a = 1.0f;
#endif

	float2 minLight = PSInput.uv1; minLight.x = pow(minLight.x, 2.8);

	float shY = 1.0 - smoothstep(0.0, 1.0, (PSInput.uv1.y - 0.875) * 30.0);

	float3 lightClr = lerp(lerp(lidCo,
		lniCo*1.4, at.night),
		licCo*1.3, 1.0 - PSInput.uv1.y);
		if (uD(FOG_COLOR)) lightClr = lunCu;
		if (nD(FOG_COLOR)) lightClr = lneCn;
		if (eD(FOG_COLOR)) lightClr = lenCe;

	float lightSrc = lerp(lerp(lerp(lerp(min(max(dot(PSInput.uv1.x - 0.2, 0.4), 0.0), max(dot(PSInput.uv1.x, 0.6), 0.0)) * pow(PSInput.uv1.x, 2.3),
		min(max(dot(PSInput.uv1.x - 0.22, 0.54), 0.0), max(dot(PSInput.uv1.x, 0.66), 0.0)) * pow(PSInput.uv1.x, 2.25), at.night),
		min(max(dot(PSInput.uv1.x - 0.1, 0.7), 0.0), max(dot(PSInput.uv1.x, 0.8), 0.0)) * pow(PSInput.uv1.x, 0.9), uD(FOG_COLOR)),
		min(max(dot(PSInput.uv1.x - 0.1, 0.4), 0.0), max(dot(PSInput.uv1.x, 0.5), 0.0)) * pow(PSInput.uv1.x, 1.7), nD(FOG_COLOR)),
		min(max(dot(PSInput.uv1.x - 0.2, 0.5), 0.0), max(dot(PSInput.uv1.x, 0.5), 0.0)) * pow(PSInput.uv1.x, 1.8), eD(FOG_COLOR));

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
		#define ALPHA_THRESHOLD 0.05
	#else
		#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif

#if defined(BLEND)
	diffuse.a *= PSInput.color.a;
#endif

#if !defined(ALWAYS_LIT)
	diffuse = diffuse * TEXTURE_1.Sample(TextureSampler1, minLight);
#endif
	float shadow = 0.0;
#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = PSInput.color.a;
	#endif	

	diffuse.rgb *= lerp(normalize(PSInput.color.rgb), abs(PSInput.color.rgb), 0.8);
#else
	float2 uv = PSInput.color.xy;
	diffuse.rgb *= lerp(1.0f, TEXTURE_2.Sample(TextureSampler2, uv).rgb*2.0f, PSInput.color.b);
	diffuse.rgb *= PSInput.color.aaa;
	diffuse.a = 1.0f;
#endif

	float3 N = normalize(cross(ddx(PSInput.ppos.zyx), ddy(PSInput.ppos.zyx)));

#ifdef ALPHA_TEST
	shadow = clamp(-N.z*3.0, 0.0, 1.0);
#else
	shadow = lerp(shadow, 1.0, max(max(abs(N.z), -N.y), 0.0));
	shadow = lerp(shadow, -2.0, clamp(2.0 * at.dusk * abs(N.x), 0.0, 1.0));
#endif

if (PSInput.color.a == 0.0) {
	shadow = lerp(shadow, 1.0, clamp(3.0 * max(abs(N.z), -N.y), 0.0, 1.0));
	shadow = lerp(shadow, -2.0, clamp(6.0 * at.dusk * abs(N.x), 0.0, 1.0));
} else {
	shadow = lerp(shadow, 1.4, shY);
}

	diffuse.rgb *= lerp(1.0, lerp(lerp(lerp(0.85, 0.95, at.night), 0.98, at.rain), 1.0, lightSrc), shadow);

	diffuse.rgb = lerp(diffuse.rgb, diffuse.rgb + lightClr, lightSrc);

	diffuse.rgb *= uD(FOG_COLOR) ? unCu : daCo;
	diffuse.rgb *= nD(FOG_COLOR) ? neCn : daCo;
	diffuse.rgb *= eD(FOG_COLOR) ? enCe : daCo;
	diffuse.rgb *= lerp(lerp(daCo, duCo, at.dusk), niCo, at.night);

	diffuse.rgb *= uD(FOG_COLOR) ? 0.9 : 1.0;
	diffuse.rgb *= nD(FOG_COLOR) ? 1.5 : 1.0;
	diffuse.rgb *= eD(FOG_COLOR) ? 0.8 : 1.0;
	diffuse.rgb *= lerp(lerp(lerp(1.0, 0.9, at.dusk), 1.1, at.night), 0.6, at.rain);

	diffuse.rgb = rgb2tone(diffuse.rgb);
	diffuse.rgb = ColorBoost(diffuse.rgb, ValueBoost);
	diffuse = rgb2hdr(diffuse);
	diffuse = smoothstep(0.0, 1.0, diffuse);

#ifdef FOG
	diffuse.rgb = lerp(diffuse.rgb, PSInput.fogColor.rgb, PSInput.fogColor.a);
#endif

	diffuse.rgb = lerp(diffuse.rgb, PSInput.fogCoIor.rgb, PSInput.fogCoIor.a);

	PSOutput.color = diffuse;

#ifdef VR_MODE
	// On Rift, the transition from 0 brightness to the lowest 8 bit value is abrupt, so clamp to 
	// the lowest 8 bit value.
	PSOutput.color = max(PSOutput.color, 1 / 255.0f);
#endif

#endif // BYPASS_PIXEL_SHADER
}